#!/usr/bin/wishx
#############################################
# wgetsession
# provides logging of a wget download session
# by Sergio Nasi - May 1999
#############################################

# set title
wm title . "wgetsession"

# catches destroy window event 
wm protocol . WM_DELETE_WINDOW ExitWgetSession

# window cannot be resized
wm resizable . 0 0

global vCommand vSessionState vMyPid vWgetPipe

set vCommand "wget $argv"

#################
# PROCEDURE: Init
#################
proc Init {} {

	global vCommand vSessionState vMyPid

	# initialize variables
	set vSessionState 0
	# get pid
	set vMyPid [pid]

	# declare widgets
	frame .top
	frame .center
	frame .down

	text .tCmdLine -relief sunken -height 3 -yscrollcommand ".sCmdLine set" \
						-bg lightyellow 
	text .text -relief raised -bd 2 -yscrollcommand ".scroll set" -bg lightgreen
	
	scrollbar .sCmdLine -command ".tCmdLine yview"
	scrollbar .scroll -command ".text yview"
	button .b_run -text Go -bg green -fg black -command RunWgetSession
	button .b_exit -text Exit -bg red -fg white -command ExitWgetSession 
	label .lState -text "IDLE" -relief ridge -bg green -fg black
	
	.text configure -state disabled
	
	# display wget command line
   .tCmdLine insert end $vCommand
   .tCmdLine see end
   .tCmdLine configure -state disabled
	

}

#########################
# PROCEDURE: PackWidgets
#########################
proc PackWidgets {} {
	
	pack .down -side bottom -fill x
	pack .top -side top -fill x
	pack .center -side top -fill x 
	
	pack .tCmdLine -in .top -side left
	pack .sCmdLine -in .top -side right -fill y
	
	pack .b_run -in .center -side left -padx 50
	pack .lState -in .center -side left  -padx 120
	pack .b_exit -in .center -side right -padx 50
	
	pack .text -in .down -side left
	pack .scroll -in .down -side right -fill y
}

###########################
# PROCEDURE: RunWgetSession 
###########################
proc RunWgetSession {} {
	
	global vCommand vWgetPipe vSessionState 
	
	# returns if state already active
	if {$vSessionState==0} {
		
		# try opening the pipe
		if {[catch {open "|$vCommand |& cat"} vWgetPipe]} {
			catch {close $vWgetPipe}
			ExitWgetSession
	
		} else {
			set vSessionState 1 
			
			# set state label
			.lState configure -text "ACTIVE" -bg red -fg white 
			
			# buffer the pipe output as line and put the pipe in non-blocking mode 
			fconfigure $vWgetPipe -buffering line -blocking 0
			
			# as soon as the pipe is open, read the output and logs it
			fileevent $vWgetPipe readable Log
		}

	}

}
		 	

############################
# PROCEDURE: ExitWgetSession 
############################
proc ExitWgetSession {} {
	

	global vCommand vSessionState vMyPid vWgetPipe
	
	# if pipe is still active, get the pid's
	if {$vSessionState==1} {
		set lvPipePids [pid $vWgetPipe]
		set lvPid0 [lindex $lvPipePids 0 ]
		set lvPid1 [lindex $lvPipePids 1 ]
		# kill each process in the pipe to avoid zombies
		catch {exec kill $lvPid0}
		catch {exec kill $lvPid1}
	}

	# finally close the pipe
	catch {close $vWgetPipe}

	# inform parent process to get rid of the zombie process
	catch {send -async tkwget.tcl KillZombie $vMyPid }
	
	exit 0

}
	

################
# PROCEDURE: Log 
################
proc Log {} {

	global vWgetPipe vSessionState
	
	# close the pipe in the event of EndOfFile
	if [eof $vWgetPipe] {
		catch {close $vWgetPipe}
		set vSessionState 0
		.lState configure -text "IDLE" -bg green -fg black 

	} else {
		# gets each line from pipe's output
		gets $vWgetPipe lvLine
		
		# if line's not empty, print it
		if {$lvLine!=""} { Print $lvLine }
		
		# update display
		update idletasks
	}
	
}


##################
# PROCEDURE: Print 
##################
proc Print aMsg {
	
	# a quick function to fill the text widget and leave it disabled
	.text configure -state normal 
	.text insert end $aMsg\n
	.text see end
	.text configure -state disabled

}


proc debug msg {

	puts stdout $msg

}

Init
PackWidgets
